@@ -7,16 +7,13 @@ use crossterm::tty::IsTty;
 use crate::util::{Point, Size};
 use std::io::{self, Write};
 
-// TODO: maybe do flushing on drop
-
 // TODO: add `error` to abort program with message?
 
 // TODO: return a result instead of `expect`ing?
 
 // Once https://github.com/rust-lang/rust/pull/78515 is merged, some of this can be changed
-pub struct Terminal {
-    // This cannot be an `io::StdoutLock` and permanently lock because that doesn't work well with threads
-    pub stdout: io::Stdout,
+pub struct Terminal<'a> {
+    pub stdout: io::BufWriter<io::StdoutLock<'a>>,
     pub size: Size,
     #[cfg(debug_assertions)]
     pub flush_count: usize,
@@ -24,26 +21,24 @@ pub struct Terminal {
     initialized: bool,
 }
 
-// TODO: for better panicking: https://play.rust-lang.org/?version=stable&mode=debug&edition=2018&gist=018af4b20094fd7ec0d4cca21d6ff2a8
-// NOTE: maybe add a panic hook (but check if there is another one already) or a Drop implementation to improve panic output
-
 #[derive(Debug)]
 #[non_exhaustive] // Prevent instantiation
 pub struct NotTTY;
 
 /// A terminal with an `io::Stdout` inside.
 ///
-/// Optimally every program should have a single instance.
-impl Terminal {
-    pub fn new() -> Result<Self, NotTTY> {
-        let stdout = io::stdout();
-
+/// Every program can have only a single instance for writing.
+/// The standard output stream is locked and no other instance can write.
+impl<'a> Terminal<'a> {
+    pub fn new(stdout: io::StdoutLock<'a>) -> Result<Self, NotTTY> {
         if !stdout.is_tty() {
             return Err(NotTTY);
         }
 
+        let writer = io::BufWriter::new(stdout);
+
         Ok(Self {
-            stdout: io::stdout(),
+            stdout: writer,
             size: Self::size(),
             #[cfg(debug_assertions)]
             flush_count: 0,
@@ -94,6 +89,22 @@ impl Terminal {
         }
     }
 
+    fn set_panic_hook() {
+        use std::panic;
+
+        let default_panic_hook = panic::take_hook();
+
+        panic::set_hook(Box::new(move |panic_info| {
+            let stdout = io::stdout();
+            if let Ok(mut terminal) = Terminal::new(stdout.lock()) {
+                terminal.deinitialize();
+            }
+            default_panic_hook(panic_info);
+        }));
+    }
+
+    /// Makes the terminal ready for drawing and input
+    /// and registers a panic hook that makes sure [`deinitialize`] is called before the panic output.
     pub fn initialize(&mut self) {
         self.enter_alternate_dimension();
         self.enable_raw_mode();
@@ -101,6 +112,8 @@ impl Terminal {
         self.hide_cursor();
         self.flush();
 
+        Self::set_panic_hook();
+
         #[cfg(debug_assertions)]
         {
             self.initialized = true;
