@@ -6,10 +6,11 @@ use crate::{
     Terminal,
 };
 use crossterm::{cursor, event, style, terminal, QueueableCommand};
+use std::time::Duration;
 
 // TODO: return result instead of unwrapping?
 
-impl Terminal {
+impl<'a> Terminal<'a> {
     pub fn enter_alternate_dimension(&mut self) {
         self.stdout.queue(terminal::EnterAlternateScreen).unwrap();
     }
@@ -44,7 +45,7 @@ impl Terminal {
 
     /// Reads an event. It also sets the new size if the terminal has been resized, hence a mutable borrow of `self` is required.
     pub fn read_event(&mut self) -> Option<Event> {
-        let crossterm_event = Terminal::read();
+        let crossterm_event = crossterm::event::read().unwrap();
         let event = match crossterm_event {
             event::Event::Mouse(event) => match event.kind {
                 event::MouseEventKind::Moved => Event::Mouse(MouseEvent {
@@ -112,7 +113,6 @@ impl Terminal {
                 }),
             },
             event::Event::Key(event::KeyEvent { code, modifiers }) => match code {
-                event::KeyCode::Tab => Event::Key(KeyEvent::Tab),
                 event::KeyCode::Char('w') if modifiers == event::KeyModifiers::CONTROL => {
                     Event::Key(KeyEvent::Backspace(Some(KeyModifier::Control)))
                 }
@@ -123,19 +123,15 @@ impl Terminal {
                         Event::Key(KeyEvent::Char(key, None))
                     }
                 }
-                event::KeyCode::Esc => Event::Key(KeyEvent::Esc),
-                event::KeyCode::Backspace => Event::Key(KeyEvent::Backspace(None)),
-                event::KeyCode::Left if modifiers == event::KeyModifiers::CONTROL => {
-                    Event::Key(KeyEvent::Left(Some(KeyModifier::Control)))
-                }
-                event::KeyCode::Right if modifiers == event::KeyModifiers::CONTROL => {
-                    Event::Key(KeyEvent::Right(Some(KeyModifier::Control)))
-                }
+                event::KeyCode::Left => Event::Key(KeyEvent::Left),
+                event::KeyCode::Right => Event::Key(KeyEvent::Right),
                 event::KeyCode::Up => Event::Key(KeyEvent::Up),
                 event::KeyCode::Down => Event::Key(KeyEvent::Down),
-                event::KeyCode::Left => Event::Key(KeyEvent::Left(None)),
-                event::KeyCode::Right => Event::Key(KeyEvent::Right(None)),
+                event::KeyCode::Tab => Event::Key(KeyEvent::Tab),
                 event::KeyCode::Enter => Event::Key(KeyEvent::Enter),
+                event::KeyCode::F(number) => Event::Key(KeyEvent::F(number)),
+                event::KeyCode::Backspace => Event::Key(KeyEvent::Backspace(None)),
+                event::KeyCode::Esc => Event::Key(KeyEvent::Esc),
                 _ => return None,
             },
             event::Event::Resize(width, height) => {
@@ -146,6 +142,14 @@ impl Terminal {
         Some(event)
     }
 
+    pub fn poll_event(&mut self, timeout: Duration) -> Option<Event> {
+        if let Ok(true) = crossterm::event::poll(timeout) {
+            self.read_event()
+        } else {
+            None
+        }
+    }
+
     pub fn set_cursor(&mut self, point: Point) {
         self.stdout.queue(cursor::MoveTo(point.x, point.y)).unwrap();
     }
@@ -188,6 +192,43 @@ impl Terminal {
             .unwrap();
     }
 
+    //
+    // TODO for the following methods: Do they work on Windows?
+    //
+
+    // Reference: https://invisible-island.net/xterm/ctlseqs/ctlseqs.html#h3-Operating-System-Commands
+    // NOTE: clipboard functionality can be added: https://github.com/alacritty/alacritty/blob/3e867a056018c507d79396cb5c5b4b8309c609c2/alacritty_terminal/src/ansi.rs#L440
+
+    /// Changes the terminal's foreground text color to `hex_color`.
+    ///
+    /// `hex_color` must be a hexadecimal color such as `"FF0000"`.
+    pub fn change_foreground_color(&mut self, hex_color: &str) {
+        self.write(&format!("\u{1b}]10;#{}\u{7}", hex_color));
+    }
+    pub fn reset_foreground_color(&mut self) {
+        self.write("\u{1b}]110\u{7}");
+    }
+
+    /// Changes the terminal's background text color to `hex_color`.
+    ///
+    /// `hex_color` must be a hexadecimal color such as `"FF0000"`.
+    pub fn change_background_color(&mut self, hex_color: &str) {
+        self.write(&format!("\u{1b}]11;#{}\u{7}", hex_color));
+    }
+    pub fn reset_background_color(&mut self) {
+        self.write("\u{1b}]111\u{7}");
+    }
+
+    /// Changes the terminal's cursor color to `hex_color`.
+    ///
+    /// `hex_color` must be a hexadecimal color such as `"FF0000"`.
+    pub fn change_cursor_color(&mut self, hex_color: &str) {
+        self.write(&format!("\u{1b}]12;#{}\u{7}", hex_color));
+    }
+    pub fn reset_cursor_color(&mut self) {
+        self.write(&format!("\u{1b}]112\u{7}"));
+    }
+
     pub fn enable_italic(&mut self) {
         self.write(&format!("{}", style::Attribute::Italic));
     }
@@ -237,8 +278,4 @@ impl Terminal {
         let size = terminal::size().expect("retrieving terminal size failed");
         Size::new(size.0, size.1)
     }
-
-    pub fn read() -> event::Event {
-        crossterm::event::read().expect("reading event failed")
-    }
 }
